@@ -2,26 +2,29 @@
 # Deutscher Wetterdienst R Daten download Klimastationen
 # Weather Data Germany download with R, Climate Data Germany
 #
-#' Download data from the DWD CDC FTP Server
-#' 
-#' Get climate data from the German Weather Service (DWD) FTP-server.
-#' The desired .zip (or .txt) dataset is downloaded into \code{dir}.
-#' If \code{read=TRUE}, it is also read, processed and returned as a data.frame.\cr
-#' To solve "errors in download.file: cannot open URL", see 
-#' \url{https://bookdown.org/brry/rdwd/station-selection.html#fileindex}.\cr
+#' @title Download data from the DWD CDC FTP Server
+#' @description Get climate data from the German Weather Service (DWD) FTP-server.
+#' The desired dataset is downloaded into `dir`.
+#' If `read=TRUE`, it is also read and processed.\cr
+#' `dataDWD` handles vectors of URLs,
+#' displays progress bars (if the package `pbapply` is available)
+#' and by default does not re-download data already in `dir`
+#' (but see argument `force` to update files).\cr
+#' To solve "errors in download.file: cannot open URL", see
+#' <https://bookdown.org/brry/rdwd/fileindex.html>.\cr
 #' 
 #' @return Presuming downloading and processing were successful:
-#'         if \code{read=TRUE}, a data.frame of the desired dataset
-#'         (as returned by \code{\link{readDWD}}),
+#'         if `read=TRUE`, the desired dataset
+#'         (as returned by [readDWD()]),
 #'         otherwise the filename as saved on disc
-#'         (may have "_n" appended in name, see \code{\link{newFilename}}).\cr
-#'         If length(file)>1, the output is a list of data.frames / vector of filenames.\cr
+#'         (may have "_n" appended in name, see [newFilename()]).\cr
+#'         If length(file)>1, the output is a list of outputs / vector of filenames.\cr
 #'         The output is always invisible.
 #' @author Berry Boessenkool, \email{berry-b@@gmx.de}, Jun-Oct 2016
-#' @seealso \code{\link{selectDWD}}. \code{\link{readDWD}}, \code{\link{download.file}}.\cr
-#'          \url{https://bookdown.org/brry/rdwd}\cr
-#'          Helpful for plotting: \code{berryFunctions::\link[berryFunctions]{monthAxis}},
-#'          see also \code{berryFunctions::\link[berryFunctions]{climateGraph}}
+#' @seealso [selectDWD()]. [readDWD()], [download.file()].\cr
+#'          <https://bookdown.org/brry/rdwd>\cr
+#'          Helpful for plotting: [berryFunctions::monthAxis()],
+#'          see also [berryFunctions::climateGraph()]
 #' @keywords data file
 #' @importFrom utils tail download.file browseURL
 #' @importFrom berryFunctions newFilename owa traceCall truncMessage
@@ -65,78 +68,85 @@
 #' # browseURL("https://bookdown.org/brry/rdwd")
 #' }
 #' 
-#' @param file   Char (vector): complete file URL(s) (including base and filename.zip) as returned by
-#'               \code{\link{selectDWD}}. Can be a vector with several filenames.
+#' @param url    Char (vector): complete file URL(s) (including base and filename.zip) 
+#'               as returned by [selectDWD()]. Can be a vector with several FTP URLs.
 #' @param base   Single char: base URL that will be removed from output file names.
-#'               DEFAULT: \code{\link{dwdbase}}
-#' @param joinbf Logical: paste \code{base} and \code{file} together? 
+#'               DEFAULT: [`dwdbase`]
+#' @param joinbf Logical: paste `base` and `file url` together?
+#'               Needed mostly for data at [`gridbase`].
 #'               DEFAULT: FALSE (selectDWD returns complete URLs already)
 #' @param dir    Char: Writeable directory name where to save the downloaded file.
-#'               Created if not existent. DEFAULT: "DWDdata" at current \code{\link{getwd}()}
-#' @param force  Logical (vector): always download, even if the file already exists in \code{dir}?
+#'               Created if not existent. DEFAULT: "DWDdata" at current [getwd()]
+#' @param force  Logical (vector): always download, even if the file already exists in `dir`?
 #'               Use NA to force re-downloading files older than 24 hours.
 #'               Use a numerical value to force after that amount of hours.
-#'               Note you might want to set \code{overwrite=TRUE} as well.
-#'               If FALSE, the file is still read (or name returned). DEFAULT: FALSE
+#'               Note: if `force!=FALSE`, you might want to set `overwrite=TRUE` as well.
+#'               If `force=FALSE`, the file is still read (or name returned).
+#'               DEFAULT: FALSE
 #' @param overwrite Logical (vector): if force=TRUE, overwrite the existing file
 #'               rather than append "_1"/"_2" etc to the filename? DEFAULT: FALSE
-#' @param dbin   Logical: Download binary file, i.e. add \code{mode="wb"} to the
-#'               \code{\link{download.file}} call? This is needed for .tar files 
-#'               (see \code{\link{readDWD.asc}}) and binary files like those at 
-#'               \href{https://opendata.dwd.de/weather/radar/radolan/rw/}{weather/radar/radolan}.
+#' @param read   Logical: read the file(s) with [readDWD()]? If FALSE,
+#'               only download is performed and the filename(s) returned. DEFAULT: TRUE
+#' @param dbin   Logical: Download binary file, i.e. add `mode="wb"` to the
+#'               [download.file()] call? This is needed for .tar files
+#'               (see [readDWD.asc()]) and binary files like those at
+#'               [weather/radar/radolan](https://opendata.dwd.de/weather/radar/radolan/rw/).
 #'               This seems to be a CRLF issue on MS Windows.
 #'               DEFAULT: FALSE
+#' @param dfargs Named list of additional arguments passed to [download.file()]
+#'               Note that mode="wb" is already passed if `dbin=TRUE`
 #' @param sleep  Number. If not 0, a random number of seconds between 0 and
-#'               \code{sleep} is passed to \code{\link{Sys.sleep}} after each download
-#'               to avoid getting kicked off the FTP-Server. DEFAULT: 0
-#' @param quiet  Logical: suppress message about directory / filenames? DEFAULT: FALSE
+#'               `sleep` is passed to [Sys.sleep()] after each download
+#'               to avoid getting kicked off the FTP-Server,
+#'               see note in [indexFTP()]. DEFAULT: 0
 #' @param progbar Logical: present a progress bar with estimated remaining time?
 #'               If missing and length(file)==1, progbar is internally set to FALSE.
-#'               Only works if the R package \code{pbapply} is available. DEFAULT: TRUE (!quiet)
-#' @param browse Logical: open repository via \code{\link{browseURL}} and
+#'               Only works if the R package `pbapply` is available. DEFAULT: TRUE (!quiet)
+#' @param browse Logical: open repository via [browseURL()] and
 #'               return URL folder path? If TRUE, no data is downloaded.
 #'               If file has several values, only unique folders will be opened.
 #'               DEFAULT: FALSE
-#' @param read   Logical: read the file(s) with \code{\link{readDWD}}? If FALSE,
-#'               only download is performed and the filename(s) returned. DEFAULT: TRUE
 #' @param ntrunc Single integer: number of filenames printed in messages
 #'               before they get truncated with message "(and xx more)". DEFAULT: 2
-#' @param dfargs Named list of additional arguments passed to \code{\link{download.file}}
-#'               Note that mode="wb" is already passed if \code{dbin=TRUE}
-#' @param \dots  Further arguments passed to \code{\link{readDWD}}, 
-#'               like fread, varnames etc. Dots were passed to 
-#'               \code{\link{download.file}} prior to rdwd 0.11.7 (2019-02-25)
+#' @param file   Deprecated since rdwd version 1.3.34, 2020-07-28.
+#' @param quiet  Logical: suppress message about directory / filenames?
+#'               DEFAULT: FALSE through [rdwdquiet()]
+#' @param \dots  Further arguments passed to [readDWD()],
+#'               like fread, varnames etc. Dots were passed to
+#'               [download.file()] prior to rdwd 0.11.7 (2019-02-25)
 #
 dataDWD <- function(
-file,
+url,
 base=dwdbase,
 joinbf=FALSE,
 dir="DWDdata",
 force=FALSE,
 overwrite=FALSE,
+read=TRUE,
 dbin=FALSE,
+dfargs=NULL,
 sleep=0,
-quiet=FALSE,
 progbar=!quiet,
 browse=FALSE,
-read=TRUE,
 ntrunc=2,
-dfargs=NULL,
+file=NULL,
+quiet=rdwdquiet(),
 ...
 )
 {
-if(!is.atomic(file)) stop("file must be a vector, not a ", class(file))
-if(!is.character(file)) stop("file must be char, not ", class(file))
+if(!is.null(file)) stop("The argument 'file' has been renamed to 'url' with rdwd version 1.3.34, 2020-07-28")
+if(!is.atomic(url)) stop("url must be a vector, not a ", class(url))
+if(!is.character(url)) stop("url must be char, not ", class(url))
 base <- sub("/$","",base) # remove accidental trailing slash
-file <- sub("^/","",file) # remove accidental leading slash
-if(joinbf)  file <- paste0(base,"/",file)
-if(missing(progbar) & length(file)==1) progbar <- FALSE
-if(any(file==""))
+url <- sub("^/","",url) # remove accidental leading slash
+if(joinbf)  url <- paste0(base,"/",url)
+if(missing(progbar) & length(url)==1) progbar <- FALSE
+if(any(url==""))
 {
-  message(traceCall(1, "", ": "), "Removing ", sum(file==""), " empty element(s) from file vector.")
-  file <- file[file!=""]
+  message(traceCall(1, "", ": "), "Removing ", sum(url==""), " empty element(s) from url vector.")
+  url <- url[url!=""]
 }
-if(length(file)<1) stop("The vector of files to be downloaded is empty.")
+if(length(url)<1) stop("The vector of urls to be downloaded is empty.")
 # be safe from accidental vector input:
 dir     <- dir[1]
 progbar <- progbar[1]
@@ -148,7 +158,7 @@ browse  <- browse[1]
 # open URL path(s) in internet browser:
 if(browse)
   {
-  folders <- unique(dirname(file))
+  folders <- unique(dirname(url))
   sapply(folders, browseURL)
   return(folders)
   }
@@ -156,7 +166,7 @@ if(browse)
 owd <- dirDWD(dir, quiet=quiet)
 on.exit(setwd(owd))
 # output file name(s)
-outfile <- gsub(paste0(base,"/"), "", file)
+outfile <- gsub(paste0(base,"/"), "", url)
 outfile <- gsub("/", "_", outfile)
 
 # force=NA management
@@ -177,20 +187,20 @@ if( any(dontdownload) & !quiet )
           berryFunctions::truncMessage(outfile[dontdownload], ntrunc=ntrunc, prefix=""),
           "\nNow downloading ",sum(!dontdownload)," files...")
   }
-outfile <- newFilename(outfile, quiet=quiet, ignore=dontdownload, 
+outfile <- newFilename(outfile, quiet=quiet, ignore=dontdownload,
                        overwrite=overwrite, ntrunc=ntrunc, tellignore=FALSE)
 # since berryFunctions 1.15.9 (2017-06-14), outfile is now an absolute path
 # Optional progress bar:
 if(progbar) lapply <- pbapply::pblapply
 # ------------------------------------------------------------------------------
 # loop over each filename
-dl_results <- lapply(seq_along(file), function(i)
+dl_results <- lapply(seq_along(url), function(i)
   if(!dontdownload[i])
   {
   # Actual file download:
-  dfdefaults <- list(url=file[i], destfile=outfile[i], quiet=TRUE)
+  dfdefaults <- list(url=url[i], destfile=outfile[i], quiet=TRUE)
   if(dbin) dfdefaults <- c(dfdefaults, mode="wb")
-  e <- try(suppressWarnings(do.call(download.file, 
+  e <- try(suppressWarnings(do.call(download.file,
                          berryFunctions::owa(dfdefaults, dfargs))), silent=TRUE)
   # wait some time to avoid FTP bot recognition:
   if(sleep!=0) Sys.sleep(runif(n=1, min=0, max=sleep))
@@ -200,17 +210,21 @@ dl_results <- lapply(seq_along(file), function(i)
 # check for download errors:
 iserror <- sapply(dl_results, inherits, "try-error")
 if(any(iserror))
-  { 
+  {
   ne <- sum(iserror)
-  msg <- paste0(ne, " Download", if(ne>1) "s have" else " has", 
-                " failed (out of ",length(iserror),").", 
-                if(read)" Setting read=FALSE.","\n")
+  msg <- paste0(ne, " Download", if(ne>1) "s have" else " has",
+                " failed (out of ",length(iserror),").",
+                if(read)" Setting read=FALSE.")
   read <- FALSE
-  msg <- paste0(msg, "download.file error",if(ne>1) "s",":\n")
+  msg <- paste0(msg, " download.file error",if(ne>1) "s",":\n")
   msg2 <- sapply(dl_results[iserror], function(e)attr(e,"condition")$message)
-  if(any(substr(file[iserror], 1, 4) != "ftp:"))
-     msg2 <- paste0(msg2, "\ndataDWD needs urls starting with 'ftp://'. You can use joinbf=TRUE for relative links.")
-  msg <- paste0(msg, paste(msg2, collapse="\n"))
+  msg2 <- berryFunctions::truncMessage(msg2, ntrunc=15, prefix="", midfix="", altnix="", sep="\n")
+  if(any(substr(url[iserror], 1, 4) != "ftp:"))
+     msg2 <- paste0(msg2, "\n- dataDWD needs urls starting with 'ftp://'. You can use joinbf=TRUE for relative links.")
+  if(grepl("cannot open URL", msg2))
+     msg2 <- paste0(msg2, "\n- If files have been renamed on the DWD server, ",
+                    "see   https://bookdown.org/brry/rdwd/fileindex.html")
+  msg <- paste0(msg, msg2)
   warning(msg, call.=FALSE)
   }
 # ------------------------------------------------------------------------------
