@@ -2,25 +2,32 @@
 #' 
 #' Create a list of all the files (in all subfolders) of an FTP server.
 #' Defaults to the German Weather Service (DWD, Deutscher WetterDienst) OpenData server at
-#' \url{ftp://opendata.dwd.de/climate_environment/CDC/observations_germany/climate/}.\cr\cr
-#' The R package \code{RCurl} must be available to do this.
-#' If \code{RCurl::\link[RCurl]{getURL}} fails, usually because bot access is
-#' detected and denied, there will still be an output which you can pass in a
-#' second run via \code{folder} to extract the remaining dirs.
-#' You might want to wait a bit and set \code{sleep} to a higher value in that case. 
-#' Here's an example:\cr
-#' \code{gridindex <- indexFTP("", gridbase)}\cr
-#' \code{gridindex <- indexFTP(gridindex, gridbase, sleep=1)}\cr
-#' 
-#' @details
+#' <ftp://opendata.dwd.de/climate_environment/CDC/observations_germany/climate/>.\cr
+#' The R package `RCurl` must be available to do this.\cr\cr
 #' It's not suggested to run this for all folders, as it can take quite some time
 #' and you may get kicked off the FTP-Server. This package contains an index
-#' of the climatic observations at weather stations: \code{View(rdwd:::\link{fileIndex})}.
-#' If it is out of date, please let me know!
+#' of the climatic observations at weather stations ([`fileIndex`]) 
+#' and gridded datasets ([`gridIndex`]).
+#' If they are out of date, please let me know!\cr\cr
+#' **Getting banned from the FTP Server**\cr
+#' Normally, this shouldn't happen anymore: since Version 0.10.10 (2018-11-26),
+#' a single RCurl handle is used for all FTP requests and since version 1.0.17 (2019-05-14),
+#' the file tree provided by the DWD is used to obtain all folders first,
+#' eliminating the recursive calls.\cr
+#' There's a provision if the FTP server detects bot requests and denies access.
+#' If [RCurl::getURL()] fails, there will still be an output
+#' which you can pass in a second run via `folder` to extract the remaining dirs.
+#' You might need to wait a bit and set `sleep` to a higher value in that case.
+#' Here's an example:\cr
+#' ```
+#' gridindex <- indexFTP("", gridbase)
+#' gridindex <- indexFTP(gridindex, gridbase, sleep=15)
+#' ```
+#' Of course, with a higher sleep value, the execution will take longer!
 #' 
 #' @return a vector with file paths
 #' @author Berry Boessenkool, \email{berry-b@@gmx.de}, Oct 2016
-#' @seealso \code{\link{createIndex}}, \code{\link{updateIndexes}}
+#' @seealso [createIndex()], [updateIndexes()]
 #' @keywords file
 #' @importFrom stats runif
 #' @importFrom pbapply pblapply
@@ -36,37 +43,41 @@
 #' }
 #' 
 #' @param folder  Folder(s) to be indexed recursively, e.g. "/hourly/wind/".
-#'                Leading slashes will be removed. 
-#'                Use \code{folder=""} to search at the location of \code{base} itself.
-#'                If \code{folder} is "currentfindex" (the default) and \code{base} 
-#'                is the default, \code{folder} is changed to all observational 
-#'                folders listed in the current tree file at 
-#'                \url{ftp://opendata.dwd.de/weather/tree.html}. With "currentgindex" 
-#'                and \code{gridbase}, the grid folders in the tree are used.
+#'                Leading slashes will be removed.
+#'                Use `folder=""` to search at the location of `base` itself.
+#'                If `folder` is "currentfindex" (the default) and `base`
+#'                is the default, `folder` is changed to all observational
+#'                folders listed in the current tree file at
+#'                <ftp://opendata.dwd.de/weather/tree.html>. With "currentgindex"
+#'                and `gridbase`, the grid folders in the tree are used.
 #'                DEFAULT: "currentfindex"
-#' @param base    Main directory of FTP server. Trailing slashes will be removed. 
-#'                DEFAULT: \code{\link{dwdbase}}
-#' @param is.file.if.has.dot Logical: if some of the input paths contain a dot, 
+#' @param base    Main directory of FTP server. Trailing slashes will be removed.
+#'                DEFAULT: [`dwdbase`]
+#' @param is.file.if.has.dot Logical: if some of the input paths contain a dot,
 #'                treat those as files, i.e. do not try to read those as if they
 #'                were a folder. Only set this to FALSE if you know what you're doing.
 #'                DEFAULT: TRUE
 #' @param exclude.latest.bin Exclude latest file at opendata.dwd.de/weather/radar/radolan?
 #'                RCurl::getURL indicates this is a pointer to the last regularly named file.
 #'                DEFAULT: TRUE
-#' @param sleep   If not 0, a random number of seconds between 0 and \code{sleep}
-#'                is passed to \code{\link{Sys.sleep}} after each read folder
-#'                to avoid getting kicked off the FTP-Server. DEFAULT: 0
+#' @param fast    Read tree file with [data.table::fread()]
+#'                (1 sec) instead of [readLines()] (10 secs)?
+#'                DEFAULT: TRUE
+#' @param sleep   If not 0, a random number of seconds between 0 and `sleep`
+#'                is passed to [Sys.sleep()] after each read folder
+#'                to avoid getting kicked off the FTP-Server, see note above. DEFAULT: 0
 #' @param dir     Writeable directory name where to save the downloaded file.
 #'                Created if not existent.
-#'                DEFAULT: "DWDdata" at current \code{\link{getwd}()}
+#'                DEFAULT: "DWDdata" at current [getwd()]
 #' @param filename Character: Part of output filename. "INDEX_of_DWD_" is prepended,
-#'                "/" replaced with "_", ".txt" appended. DEFAULT: folder[1]
+#'                "/" replaced with "_", ".txt" appended. DEFAULT: folder\[1]
 #' @param overwrite Logical: Overwrite existing file? If not, "_n" is added to the
-#'                filename, see \code{berryFunctions::\link[berryFunctions]{newFilename}}.
+#'                filename, see [berryFunctions::newFilename()].
 #'                DEFAULT: FALSE
-#' @param quiet   Suppress progbars and message about directory/files? DEFAULT: FALSE
+#' @param quiet   Suppress progbars and message about directory/files?
+#'                DEFAULT: FALSE through [rdwdquiet()]
 #' @param progbar Logical: present a progress bar in each level? DEFAULT: TRUE
-#' @param verbose Logical: write a lot of messages from \code{RCurl::\link[RCurl]{getURL}}?
+#' @param verbose Logical: write a lot of messages from [RCurl::getURL()]?
 #'                DEFAULT: FALSE (usually, you dont need all the curl information)
 #' 
 indexFTP <- function(
@@ -74,11 +85,12 @@ folder="currentfindex",
 base=dwdbase,
 is.file.if.has.dot=TRUE,
 exclude.latest.bin=TRUE,
+fast=TRUE,
 sleep=0,
 dir="DWDdata",
 filename=folder[1],
 overwrite=FALSE,
-quiet=FALSE,
+quiet=rdwdquiet(),
 progbar=!quiet,
 verbose=FALSE
 )
@@ -90,10 +102,16 @@ if(grepl("^https", base)) warning("base should start with ftp://, not https://.
 if(all(folder %in% c("currentfindex","currentgindex")) & base %in% c(dwdbase, gridbase))
   {
   if(!quiet) message("Reading current index tree file...")
-  tree <- readLines("ftp://opendata.dwd.de/weather/tree.html")
+  if(!fast)
+     {tree <- readLines("ftp://opendata.dwd.de/weather/tree.html")
+     } else {
+     checkSuggestedPackage("data.table", "rdwd::indexFTP with fast=TRUE")
+     tree <- data.table::fread("ftp://opendata.dwd.de/weather/tree.html",
+                                sep="\n", header=FALSE, showProgress=FALSE, strip.white=FALSE)$V1
+     }
   if(!quiet) message("Processing index tree file...")
   urlpart <- "\"https://opendata.dwd.de/climate_environment/CDC/observations_germany/climate/"
-  if(folder=="currentgindex") 
+  if(folder=="currentgindex")
      urlpart <- sub("observations_germany/climate", "grids_germany", urlpart)
   tree <- grep(urlpart, tree, value=TRUE)
   tree <- sapply(strsplit(tree, "href="), "[", 2)
@@ -115,7 +133,7 @@ curl_handle <- RCurl::getCurlHandle(ftp.use.epsv=TRUE)
 
 # remove trailing slashes in base and leading slashes in folder:
 while(grepl("/$", base)) base <- sub("/$", "", base)
-while(any(grepl("^/", folder))) folder <- sub("^/","",folder) 
+while(any(grepl("^/", folder))) folder <- sub("^/","",folder)
 
 # central object: df_ff (dataframe with file/folder names)
 df_ff <- data.frame(path=folder, isfile=FALSE, stringsAsFactors=FALSE)
@@ -125,10 +143,10 @@ if(is.file.if.has.dot)
 
 # List files at a single path, returning files/folders or useful error messages
 # stoppp_ffe will be an object within indexFTP, see below
-getURL_ffe <- function(ff_row) 
+getURL_ffe <- function(ff_row)
  {
  if(stoppp_ffe) return(ff_row) # do not attempt if already kicked off the FTP
- p <- try( RCurl::getURL(paste0(base,"/",ff_row$path,"/"), verbose=verbose, 
+ p <- try( RCurl::getURL(paste0(base,"/",ff_row$path,"/"), verbose=verbose,
                          ftp.use.epsv=TRUE, curl=curl_handle),      silent=TRUE)
  # upon failure, exit with a warning (not error):
  if(inherits(p, "try-error"))
@@ -136,12 +154,12 @@ getURL_ffe <- function(ff_row)
      # Prepare warning message text:
      p <- gsub("Error in function (type, msg, asError = TRUE)  : ", "", p, fixed=TRUE)
      p <- removeSpace(gsub("\n", "", p))
-     if(grepl("Could not resolve host", p)) 
+     if(grepl("Could not resolve host", p))
        p <- paste0(p,"\nThis may mean you are not connected to the internet.")
-     if(grepl("Server denied you to change to the given directory", p)) 
+     if(grepl("Server denied you to change to the given directory", p))
        p <- paste0(p,"\nThis could mean the path is a file, not a folder",
                    " or that it doesn't exist at base\n", base)
-     msg <- paste0(traceCall(3, "", ": "), "RCurl::getURL failed for '", 
+     msg <- paste0(traceCall(3, "", ": "), "RCurl::getURL failed for '",
                    ff_row$path, "/' with error:\n - ", p)
      warning(msg, call.=FALSE)
      assign("stoppp_ffe", TRUE, inherits=TRUE) # to get out of the loop sans error
@@ -153,8 +171,8 @@ getURL_ffe <- function(ff_row)
  p <- p[nchar(p)>0]
  # handle opendata.dwd.de/weather/radar/radolan/* latest data
  ilf <- grep("latest-dwd---bin", p) # index of latest file
- if(length(ilf)>0) 
-   if(exclude.latest.bin) p <- p[-ilf]  else  
+ if(length(ilf)>0)
+   if(exclude.latest.bin) p <- p[-ilf]  else
    p[ilf] <- sub(" -> .*", "", p[ilf]) # to keep p suited for read.text
  #
  isdir <- substr(p,1,1) =="d" # directory, else file
@@ -180,7 +198,7 @@ while(any(!df_ff$isfile))           # potential ToDo: exclude previously checked
   if(stoppp_ffe) break
   } # end while loop
 
-if(anyDuplicated(df_ff$path)) warning("Duplicate paths:", 
+if(anyDuplicated(df_ff$path)) warning("Duplicate paths:",
                    truncMessage(df_ff$path[duplicated(df_ff$path)], prefix=""))
 
 # sort final results alphabetically (path only, no f/f info):
