@@ -1,36 +1,43 @@
 #' @title Select data from the DWD CDC FTP Server
 #' 
-#' @description Select files for downloading with \code{\link{dataDWD}}.\cr
+#' @description Select files for downloading with [dataDWD()].\cr
 #' The available folders with datasets are listed at
-#' \url{https://bookdown.org/brry/rdwd/available-datasets.html}.
-#' To use an updated index (if necessary), see 
-#' \url{https://bookdown.org/brry/rdwd/station-selection.html#fileindex}.\cr\cr
-#' All arguments (except for \code{mindex}, \code{findex} and \code{base})
+#' <https://bookdown.org/brry/rdwd/available-datasets.html>.
+#' To use an updated index (if necessary), see
+#' <https://bookdown.org/brry/rdwd/fileindex.html>.\cr\cr
+#' All arguments (except for `mindex`, `findex` and `base`)
 #' can be a vector and will be recycled to the maximum length of all arguments.
-#' If that length > 1, the output is a list of filenames (or vector if \code{outvec=TRUE}).\cr
-#' If station \code{name} is given, but \code{id} is empty (""),
-#' \bold{id} is inferred via \code{mindex}.
-#' If \code{res/var/per} are given and valid (existing in \code{findex}),
-#' they are pasted together to form a \bold{path}.
+#' If that length > 1, the output is a list of filenames (or vector if `outvec=TRUE`).\cr
+#' If station `name` is given, but `id` is empty (""),
+#' **id** is inferred via [findID()] using `mindex`.
+#' If `res/var/per` are given and valid (existing in `findex`),
+#' they are pasted together to form a **path**.
 #' Here is an overview of the behavior in each case of availability:
 #' \tabular{llll}{
-#' case \tab |  \bold{id} \tab |  \bold{path} \tab | output \cr
-#'  1 \tab |  ""  \tab |  ""  \tab | \code{base} (and some warnings) \cr
-#'  2 \tab | "xx" \tab |  ""  \tab | All file names (across paths) for station \bold{id} \cr
-#'  3 \tab |  ""  \tab | "xx" \tab | The zip file names at \bold{path} \cr
+#' case \tab |  **id** \tab |  **path** \tab | output \cr
+#'  1 \tab |  ""  \tab |  ""  \tab | `base` (and some warnings) \cr
+#'  2 \tab | "xx" \tab |  ""  \tab | All file names (across paths) for station **id** \cr
+#'  3 \tab |  ""  \tab | "xx" \tab | The zip file names at **path** \cr
 #'  4 \tab | "xx" \tab | "xx" \tab | Regular single data file name \cr
 #' }
-#' For case 2, you can explicitly set \code{res="",var="",per=""} to avoid the 
+#' For case 2, you can explicitly set `res="",var="",per=""` to avoid the
 #' default interactive selection.\cr
-#' For case 3 and 4 (\bold{path} given), you can set \code{meta=TRUE}.
-#' Then selectDWD will return the name of the station description file at \bold{path}.
-#' This is why case 3 with \code{meta=FALSE} only returns the data file names (ending in .zip).\cr\cr\cr
+#' For case 3 and 4 (**path** given), you can set `meta=TRUE`.
+#' Then selectDWD will return the name of the station description txt file at **path**.
+#' This is why case 3 with the default `meta=FALSE` only returns the 
+#' data file names (ending in .zip) and not the description and Beschreibung txt/pdf files.
+#' Open those in a browser with
+#' ```
+#' pdfpath <- grep("daily/kl/h.*DESCRIPTION", fileIndex$path, value=TRUE)
+#' browseURL(paste0(dwdbase, "/", pdfpath))
+#' ```
+#' Let me know if besides `meta`, `pdf` is needed for automated opening.\cr\cr\cr
 #' 
 #' @return Character string with file path and name(s) in the format
 #'         "base/res/var/per/filename.zip"
 #' @author Berry Boessenkool, \email{berry-b@@gmx.de}, Oct 2016
-#' @seealso \code{\link{dataDWD}}, \code{\link{metaIndex}},
-#'          \url{https://bookdown.org/brry/rdwd}
+#' @seealso [dataDWD()], [`metaIndex`],
+#'          <https://bookdown.org/brry/rdwd>
 #' @keywords file
 #' @importFrom berryFunctions truncMessage traceCall
 #' @importFrom utils menu
@@ -67,55 +74,69 @@
 #' 
 #' }
 #' 
-#' @param name  Char: station name(s) passed to \code{\link{findID}}, along with
-#'              \code{exactmatch} and \code{mindex}. 
-#'              All 3 arguments are ignored if \code{id} is given. DEFAULT: ""
-#' @param res   Char: temporal \bold{res}olution available at \code{base}, usually one of
-#'              \code{c("hourly","daily","monthly")}, see section 'Description' above.
-#'              \code{res/var/per} together form the \bold{path}. 
+#' @param name  Char: station name(s) passed to [findID()], along with
+#'              `exactmatch` and `mindex`.
+#'              All 3 arguments are ignored if `id` is given. DEFAULT: ""
+#' @param res   Char: temporal **res**olution available at `base`, usually one of
+#'              `c("hourly","daily","monthly")`, see section 'Description' above.
+#'              `res/var/per` together form the **path**.
 #'              DEFAULT: NA for interactive selection
-#' @param var   Char: weather \bold{var}iable of interest, like e.g.
-#'              \code{"air_temperature", "cloudiness", "precipitation",
-#'                      "soil_temperature", "solar", "kl", "more_precip"}
-#'              See above and in \code{View(rdwd:::\link{fileIndex})}. 
+#' @param var   Char: weather **var**iable of interest, like e.g.
+#'              `"air_temperature", "cloudiness", "precipitation",
+#'              "soil_temperature", "solar", "kl", "more_precip"`
+#'              See above and in [`fileIndex`].
 #'              DEFAULT: NA for interactive selection
-#' @param per   Char: desired time \bold{per}iod. One of
+#' @param per   Char: desired time **per**iod. One of
 #'              "recent" (data from the last year, up to date usually within a few days) or
 #'              "historical" (long time series). Can be abbreviated (if the first
 #'              letter is "r" or "h", full names are used). To get both datasets,
-#'              use \code{per="hr"} or \code{per="rh"} (and \code{outvec=TRUE}).
-#'              \code{per} is set to "" if var=="solar". 
+#'              use `per="hr"` or `per="rh"` (and `outvec=TRUE`).
+#'              `per` is set to "" if var=="solar".
 #'              DEFAULT: NA for interactive selection
-#' @param exactmatch Logical passed to \code{\link{findID}}: match \code{name}
-#'              with \code{\link{==}})? Else with \code{\link{grepl}}. DEFAULT: TRUE
-#' @param mindex Single object: Index with metadata passed to \code{\link{findID}}.
-#'              DEFAULT: \code{rdwd:::\link{metaIndex}}
-#' @param quiet Suppress id length warnings?
+#' @param base  Single char: main directory of DWD ftp server.
+#'              Must be the same `base` used to create `findex`.
+#'              DEFAULT: [`dwdbase`]
+#' @param outvec Single logical: if **path** or **ID** length > 1,
+#'              instead of a list, return a vector? (via [unlist()]).
+#'              DEFAULT: `per %in% c("rh","hr")`
+#' @param findex Single object: Index used to select filename, as returned by
+#'              [createIndex()].To use a current / custom index, see
+#'              <https://bookdown.org/brry/rdwd/fileindex.html>. 
+#'              DEFAULT: [`fileIndex`]
+#' @param remove_dupli Logical: Remove duplicate entries in the fileIndex? 
+#'              If duplicates are found, a warning will be issued, unless `quiet=TRUE`.\cr
+#'              The DWD updates files on the server quite often and sometimes
+#'              misses removing the old files, leading to duplicates, 
+#'              usually with differences only in the date range. 
+#'              A semi-current (manually updated) list of duplicates is on 
+#'              [github](https://github.com/brry/rdwd/blob/master/misc/ExampleTests/warnings.txt).\cr
+#'              Before reporting, run [updateRdwd()] to see if [`fileIndex`] has been updated.
+#'              I email the DWD about duplicates when I find them, they usually fix it soon.\cr
+#'              If `remove_dupli=TRUE`, only the file with the longer timespan will be kept. 
+#'              This is selected according to filename, which is not very reliable,
+#'              hence manual checking is recommended.
+#'              DEFAULT: TRUE
+#' @param current Single logical for case 3/4 with given `path`: instead of
+#'              `findex`, use a list of the currently available files at
+#'              base/res/var/per? This will call [indexFTP()], thus
+#'              requires availability of the `RCurl` package.
+#'              DEFAULT: FALSE
 #' @param id    Char/Number: station ID with or without leading zeros, e.g. "00614" or 614.
 #'              Is internally converted to an integer, because some DWD meta data
-#'              files also contain no leading zeros. DEFAULT: findID(name, exaxtmatch, mindex)
-#' @param base  Single char: main directory of DWD ftp server.
-#'              Must be the same \code{base} used to create \code{findex}.
-#'              DEFAULT: \code{\link{dwdbase}}
-#' @param findex Single object: Index used to select filename, as returned by
-#'              \code{\link{createIndex}}.To use a current / custom index, use
-#'              \code{myIndex <- createIndex(indexFTP("/daily/solar"))}
-#'              (with desired path, of course). DEFAULT: \code{rdwd:::\link{fileIndex}}
-#' @param current Single logical for case 3/4 with given \code{path}: instead of
-#'              \code{findex}, use a list of the currently available files at
-#'              base/res/var/per? This will call \code{\link{indexFTP}}, thus
-#'              requires availability of the \code{RCurl} package.
-#'              DEFAULT: FALSE
+#'              files also contain no leading zeros. 
+#'              DEFAULT: findID(name, exaxtmatch, mindex)
+#' @param mindex Single object: Index with metadata passed to [findID()].
+#'              DEFAULT: [`metaIndex`]
+#' @param exactmatch Logical passed to [findID()]: match `name`
+#'              with [`==`])? Else with [grepl()]. DEFAULT: TRUE
 #' @param meta  Logical: return metadata txt file name instead of climate data zip file?
 #'              Relevant only in case 4 (path and id given) and case 3 for res="multi_annual".
-#'              See \code{\link{metaIndex}} for a compilation of all metaData files.
+#'              See [`metaIndex`] for a compilation of all metaData files.
 #'              DEFAULT: FALSE
-#' @param meta_txt_only Logical: if \code{meta}, only return .txt files, not the 
+#' @param meta_txt_only Logical: if `meta`, only return .txt files, not the
 #'              pdf and html files? DEFAULT: TRUE
-#' @param outvec Single logical: if \bold{path} or \bold{ID} length > 1,
-#'              instead of a list, return a vector? (via \code{\link{unlist}}).
-#'              DEFAULT: \code{per \%in\% c("rh","hr")}
-#' @param \dots Further arguments passed to \code{\link{indexFTP}} if \code{current=TRUE},
+#' @param quiet Suppress id length warnings? DEFAULT: FALSE through [rdwdquiet()]
+#' @param \dots Further arguments passed to [indexFTP()] if `current=TRUE`,
 #'              except folder and base.
 #' 
 selectDWD <- function(
@@ -123,16 +144,17 @@ name="",
 res=NA,
 var=NA,
 per=NA,
-exactmatch=TRUE,
-mindex=metaIndex,
-quiet=FALSE,
-id=findID(name, exactmatch=exactmatch, mindex=mindex, quiet=quiet),
 base=dwdbase,
+outvec=any(per %in% c("rh","hr")),
 findex=fileIndex,
+remove_dupli=TRUE,
 current=FALSE,
+id=findID(name, exactmatch=exactmatch, mindex=mindex, quiet=quiet),
+mindex=metaIndex,
+exactmatch=TRUE,
 meta=FALSE,
 meta_txt_only=TRUE,
-outvec=any(per %in% c("rh","hr")),
+quiet=rdwdquiet(),
 ...
 )
 {
@@ -161,7 +183,7 @@ if(any(per=="hr"|per=="rh", na.rm=TRUE))
 # recycle input vectors
 len <- max(length(id), length(res), length(var), length(per), length(meta)  )
 lmin <-  c(length(id), length(res), length(var), length(per), length(meta)  )
-if(any(lmin==0)) stop(sum(lmin==0), " input vectors have length zero: ", 
+if(any(lmin==0)) stop(sum(lmin==0), " input vectors have length zero: ",
                       toString(c("id","res","var","per","meta")[lmin==0]))
 # outside of the loop, the slowest part of the code is getting length(id)
 # because findID obtains id from name. All computing time happens in tolower
@@ -185,10 +207,10 @@ per[var=="solar" & res %in% c("hourly","daily")] <- ""
 rma <- res=="multi_annual"
 if(any(rma, na.rm=TRUE))
   {
-  if(any(id[rma]!="", na.rm=TRUE)) warning(traceCall(1, "", ": "), "multi_annual data is not ", 
+  if(any(id[rma]!="", na.rm=TRUE)) warning(traceCall(1, "", ": "), "multi_annual data is not ",
       "organized by station ID. Setting id to ''.", call.=FALSE)
   id[rma] <- ""
-  if(any(per[rma]!="", na.rm=TRUE)) warning(traceCall(1, "", ": "), "multi_annual data is not ", 
+  if(any(per[rma]!="", na.rm=TRUE)) warning(traceCall(1, "", ": "), "multi_annual data is not ",
       "organized in period folders. Setting per to ''.", call.=FALSE)
   per[rma] <- ""
   }
@@ -218,7 +240,7 @@ selectPrompt <- function(column, RES="", VAR="", PER="", ID="", index=findex)
  VAR[is.na(VAR)] <- ""
  PER[is.na(PER)] <- ""
  if(all(VAR=="solar" & RES %in% c("hourly","daily"))) return("")
- question <- if(column=="res") "resolutions" else 
+ question <- if(column=="res") "resolutions" else
              if(column=="var") "variables"   else "periods"
  question <- paste("Which of the following", question, "would you like to use?")
  sid  <- if(any(ID !="")) index$id  %in% ID  else TRUE
@@ -227,7 +249,7 @@ selectPrompt <- function(column, RES="", VAR="", PER="", ID="", index=findex)
  sper <- if(any(PER!="")) index$per %in% PER else TRUE
  options <- index[sid & sres & svar & sper , column]
  options <- sort(unique(options))
- if(length(options)<1) 
+ if(length(options)<1)
    {
    warning("For interactive selection, no options were found for your request. ",
            "Proceeding with ", column,"=''.", call.=FALSE)
@@ -249,10 +271,11 @@ selectPrompt <- function(column, RES="", VAR="", PER="", ID="", index=findex)
 if(anyNA(res)) res[is.na(res)] <- selectPrompt("res", res, var, per, id)
 if(anyNA(var)) var[is.na(var)] <- selectPrompt("var", res, var, per, id)
 if(anyNA(per)) per[is.na(per)] <- selectPrompt("per", res, var, per, id)
-# 
+#
 # ------------------------------------------------------------------------------
 #
 # loop over each input element:
+outwarn <- new.env() # collect all warnings from the loop
 output <- lapply(seq_len(len), function(i)
 {
 # ------------------------------------------------------------------------------
@@ -265,7 +288,7 @@ if(res[i]=="multi_annual") givenpath <- res[i]!=""
 # base + warning
 if(!givenid & !givenpath)
   {
-  warning(traceCall(3, "", ": "), "neither station ID nor valid FTP folder is given.", call.=FALSE)
+  outwarn$idpath <- "neither station ID nor valid FTP folder is given."
   # call.=FALSE to avoid uninformative  Error in FUN(X[[i]], ...) :
   return(base)
   }
@@ -273,16 +296,16 @@ if(!givenid & !givenpath)
 # all file names for station ID, regardless of path
 if(givenid & !givenpath)
   {
-  if(meta[i]) warning(traceCall(3, "", ": "),
-          "meta is ignored if id is given, but path is not given.", call.=FALSE)
+  if(meta[i]) outwarn$idpathmeta <- "meta is ignored if id is given, but path is not given."
   filename <- findex[findex$id %in% id[i], "path"]
   filename <- filename[!is.na(filename)]
   # check output length
-  if(length(filename)<1 & !quiet) warning(traceCall(3, "", ": "), "in file index '", 
-     findexname, "', no filename could be detected with ID ", id[i], ".", call.=FALSE)
-  if(length(filename)>1 & !quiet) warning(traceCall(3, "", ": "), "in file index '", 
-     findexname, "', there are ", length(filename), " files with ID ",id[i], ".", 
-     call.=FALSE)
+  if(length(filename)!=1 & !quiet) 
+    {
+    outwarn$id_non1_msg <- paste0("in file index '", findexname, "', there are ")
+    outwarn$id_non1_len <- c(outwarn$id_non1_len, length(filename))
+    outwarn$id_non1_ids <- c(outwarn$id_non1_ids, id[i])
+    }
   return(   paste0(base,"/",filename)   )
   }
 #
@@ -292,8 +315,11 @@ if(res[i]=="multi_annual" & per[i]=="") {per[i] <- var[i]; var[i] <- ""}
 # select entries from file index:
 sel <- res[i]==findex$res & var[i]==findex$var & per[i]==findex$per
 # within the loop, computing time seems to come from the selection
-if(sum(sel)<1) warning(traceCall(3, "", ": "), "according to file index '",
-           findexname, "', the path '", path, "' doesn't exist.", call.=FALSE)
+if(sum(sel)<1)  
+    {
+    outwarn$miss_path_msg <- paste0("according to file index '", findexname, "', the following ") # paths don't exist
+    outwarn$miss_path_pth <- c(outwarn$miss_path_pth, path)
+    }
 #
 # case 3 or 4 with meta=TRUE
 # return name of description txt file
@@ -302,8 +328,11 @@ if(meta[i])
   sel <- sel & findex$ismeta
   if(meta_txt_only[i]) sel <- sel & ! grepl(".pdf$", findex$path)
   # checks:
-  if(sum(sel)==0) warning(traceCall(3, "", ": "), "according to file index '",findexname,
-                     "', there is no description file in '", path, "'.", call.=FALSE)
+  if(sum(sel)==0) 
+    {
+    outwarn$miss_desc_msg <- paste0("according to file index '", findexname, "', there is no description file in ")
+    outwarn$miss_desc_pth <- c(outwarn$miss_desc_pth, path)
+    }
   filename <- findex[sel,"path"]
   return(   paste0(base,"/",filename)   )
   }
@@ -313,11 +342,14 @@ if(!givenid & givenpath & !meta[i])
   {
   isnotmeta <- grepl('.zip$', findex$path)
   if(res[i]=="multi_annual") isnotmeta <- !findex$ismeta
-  sel <- sel & isnotmeta 
+  sel <- sel & isnotmeta
   filename <- findex[sel,"path"]
-  if(length(filename)<1) warning(traceCall(3, "", ": "), "according to file index '",
-                                 findexname, "', there is no file in '", path,
-                                 "' with ID ", id[i], ".", call.=FALSE)
+  if(length(filename)<1)
+    {
+    outwarn$miss_file_msg <- paste0("according to file index '", findexname, "', there is no file in")
+    outwarn$miss_file_pth[[path]] <- c(outwarn$miss_file_pth[[path]], id[i])
+    return(base)
+    }
   return(   paste0(base,"/",filename)   )
   }
 # 4: id and path are both given ------------------------------------------------
@@ -325,10 +357,24 @@ if(!givenid & givenpath & !meta[i])
 if(givenid & givenpath & !meta[i])
   {
   sel <- sel & findex$id %in% id[i]
-  if(sum(sel)==0) warning(traceCall(3, "", ": "), "according to file index '",findexname,
-                          "', there is no file in '", path, "' with ID ",
-                          id[i], ".", call.=FALSE)
-  filename <- findex[sel,"path"]
+  if(sum(sel)==0)
+    {
+    outwarn$miss_file_msg <- paste0("according to file index '", findexname, "', there is no file in")
+    outwarn$miss_file_pth[[path]] <- c(outwarn$miss_file_pth[[path]], id[i])
+    return(base)
+    }
+  if(sum(sel)>1 & remove_dupli) 
+    {
+    filename <- findex[sel,]
+    filename <- filename[which.max(filename$end - filename$start),"path"]
+    if(!quiet) warning(traceCall(3, "", ": "), "duplicate file on FTP server according to file index '", 
+            findexname, "'.\n - Proceeding with file with longest time period, ",
+            "according to time stamps in file names. Unreliable, hence:\n",
+            " - Please check actual content manually. Get all urls by setting remove_dupli=FALSE.\n",
+            " - Please report this entire warning message to berry-b@gmx.de or at ",
+            "https://github.com/brry/rdwd/issues\n - Keeping: ", filename, call.=FALSE)
+    } else
+    filename <- findex[sel,"path"]
   if(length(filename)>1) warning(traceCall(3, "", ": "), "several files (",
                                  length(filename),") were selected:",
                                  berryFunctions::truncMessage(filename, prefix=""),
@@ -336,7 +382,49 @@ if(givenid & givenpath & !meta[i])
   return(   paste0(base,"/",filename)   )
   }
 }) # loop end
+# combine all collected warnings into a single warning message:
+truncM <- function(x) berryFunctions::truncMessage(x, ntrunc=30, prefix="", midfix="", altnix="")
+if(!is.null(outwarn$id_non1_msg)) 
+  {
+  outwarn$id_non1 <- paste0(outwarn$id_non1_msg, truncM(outwarn$id_non1_len), " files with ID ", truncM(outwarn$id_non1_ids))
+  rm(id_non1_msg, envir=outwarn)
+  rm(id_non1_len, envir=outwarn)
+  rm(id_non1_ids, envir=outwarn)
+  }
+if(!is.null(outwarn$miss_path_msg)) 
+  {
+  missing_path_msg <- if(length(outwarn$miss_path_pth)>1) 
+    paste0(length(outwarn$miss_path), " paths don't exist: ") else "path doesn't exist: "
+  outwarn$miss_path <- paste0(outwarn$miss_path_msg, missing_path_msg, truncM(outwarn$miss_path_pth))
+  rm(miss_path_pth, envir=outwarn)
+  rm(miss_path_msg, envir=outwarn)
+  }
+if(!is.null(outwarn$miss_desc_msg)) 
+  {
+  outwarn$miss_desc <- paste0(outwarn$miss_desc_msg, truncM(outwarn$miss_desc_pth))
+  rm(miss_desc_pth, envir=outwarn)
+  rm(miss_desc_msg, envir=outwarn)
+  }
+if(!is.null(outwarn$miss_file_msg)) 
+  {
+  missing_file <- sapply(outwarn$miss_file_pth, truncM)
+  missing_file <- paste0("'", names(missing_file), "' with ID ", missing_file)
+  if(length(missing_file)>1) missing_file <- paste0("\n - ", missing_file)
+  outwarn$miss_file <- paste0(outwarn$miss_file_msg, " ", paste(missing_file, collapse=""), ".")
+  rm(miss_file_pth, envir=outwarn)
+  rm(miss_file_msg, envir=outwarn)
+  }
+
+outwarn <- as.list(outwarn)
+if(length(outwarn)>0 & !quiet) warning(berryFunctions::traceCall(1, "", ": "), 
+                                       paste0(outwarn, collapse="\n"), call.=FALSE)
+
 output <- if(len==1) output[[1]] else output
 if(len>1 & outvec) output <- unlist(output)
 return(output)
 }
+
+# Suppress CRAN check note 'no visible binding for global variable':
+if(getRversion() >= "2.15.1")  utils::globalVariables(c("id_non1_ids", "id_non1_len", 
+"id_non1_msg", "miss_desc_msg", "miss_desc_pth", "miss_file_msg", "miss_file_pth", 
+"miss_path_msg", "miss_path_pth"))
